@@ -22,9 +22,9 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <popt.h>
-#include <gnome-xml/tree.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 #include <stdlib.h>
 #include <errno.h>
 
@@ -56,6 +56,7 @@ static int short_docs_mode = FALSE;
 static int long_docs_mode = FALSE;
 static int schema_name_mode = FALSE;
 static int associate_schema_mode = FALSE;
+static int default_source_mode = FALSE;
 
 struct poptOption options[] = {
   { 
@@ -320,6 +321,15 @@ struct poptOption options[] = {
     N_("Specify the schema name followed by the key to apply the schema name to"),
     NULL
   },
+  {
+    "get-default-source",
+    '\0',
+    POPT_ARG_NONE,
+    &default_source_mode,
+    0,
+    N_("Get the name of the default source"),
+    NULL
+  },
   {
     NULL,
     '\0',
@@ -350,6 +360,7 @@ static int do_short_docs (GConfEngine *conf, const gchar **args);
 static int do_long_docs (GConfEngine *conf, const gchar **args);
 static int do_get_schema_name (GConfEngine *conf, const gchar **args);
 static int do_associate_schema (GConfEngine *conf, const gchar **args);
+static int do_get_default_source (GConfEngine *conf, const gchar **args);
 
 int 
 main (int argc, char** argv)
@@ -395,7 +406,7 @@ main (int argc, char** argv)
       (all_entries_mode && set_mode) ||
       (all_entries_mode && unset_mode))
     {
-      fprintf(stderr, _("Can't use --all-pairs with --get or --set\n"));
+      fprintf(stderr, _("Can't use --all-entries with --get or --set\n"));
       return 1;
     }
 
@@ -413,7 +424,7 @@ main (int argc, char** argv)
       (recursive_list && all_entries_mode) ||
       (recursive_list && all_subdirs_mode))
     {
-      fprintf(stderr, _("--recursive-list should not be used with --get, --set, --unset, --all-pairs, or --all-dirs\n"));
+      fprintf(stderr, _("--recursive-list should not be used with --get, --set, --unset, --all-entries, or --all-dirs\n"));
       return 1;
     }
 
@@ -423,7 +434,7 @@ main (int argc, char** argv)
       (set_schema_mode && all_entries_mode) ||
       (set_schema_mode && all_subdirs_mode))
     {
-      fprintf(stderr, _("--set_schema should not be used with --get, --set, --unset, --all-pairs, --all-dirs\n"));
+      fprintf(stderr, _("--set_schema should not be used with --get, --set, --unset, --all-entries, --all-dirs\n"));
       return 1;
     }  
 
@@ -545,18 +556,7 @@ main (int argc, char** argv)
           /* Properly set, but set to nothing (use default source) */
           config_source = NULL;
         }
-      
-      if (gconf_ping_daemon())
-        {
-          /* Eventually you should be able to run gconfd as long as
-             you're installing to a different database from the one
-             it's using, but I don't trust the locking right now. */
-          fprintf(stderr, _("Shouldn't run gconfd while installing new schema files.\nUse gconftool --shutdown to shut down the daemon, most safely while no applications are running\n(though things theoretically work if apps are running).\n"));
-          return 1;
-        }
 
-      /* Race condition! gconfd could start up. But we do have locking
-         as a second line of defense. */
       use_local_source = TRUE;
     }
   
@@ -717,6 +717,16 @@ main (int argc, char** argv)
           return 1;
         }
     }
+
+  if (default_source_mode)
+    {
+      const gchar** args = poptGetArgs(ctx);
+      if (do_get_default_source(conf, args)  == 1)
+        {
+          gconf_engine_unref(conf);
+          return 1;
+        }
+    }
   
   if (all_entries_mode)
     {
@@ -2496,3 +2506,36 @@ do_break_directory(GConfEngine* conf, const gchar** args)
 
   return 0;
 }
+
+static int
+do_get_default_source(GConfEngine* conf, const gchar** args)
+{
+  gchar *source;
+  gchar buf[512];
+  FILE *f;
+
+  /* Try with $sysconfdir/gconf/schema-install-source */
+  f = fopen(GCONF_ETCDIR"/schema-install-source", "r");
+
+  if (f != NULL)
+    {
+      source = fgets(buf, 512, f);
+      fclose(f);  
+      if (source)
+	{
+	  g_strchomp(source);
+	  if (*source != '\0')
+	    {
+	      printf("%s\n", source);
+	      return 0;
+	    }
+	}
+    }
+
+  /* Use default database */
+  source = g_strconcat("xml::", GCONF_ETCDIR, "/gconf.xml.defaults", NULL);
+  printf("%s\n", source);
+  g_free(source);
+
+  return 0;
+}
